import java.awt.*;
import java.awt.event.*;
import java.awt.geom.*;
import java.util.*;
import java.math.*; 
import javax.swing.*;
 
 
public class MainWindow {
    //Singleton-Muster, so dasss es nur ein Fenster gibt
    private static MainWindow canvasSingleton;

    public static MainWindow getMainWindow() {
        if(canvasSingleton == null) {
            canvasSingleton = new MainWindow();
        }
        return canvasSingleton;
    }
    
    //Attribute
    private PaintingComponent paintingComponent;
    private BinBaum myTree;
    private JFrame frame;
    private JScrollPane scrollPane;

    /**
     * Im Konstruktor wird die bliche Arbeit erledigt um den JFrame zu ffnen
     * und die Komponenten zu initialisieren
     */
    private MainWindow() {
        //Attribute definieren
        
        //Irgendwas stimmt mit dem Konstruktor wohl nicht, sonst wrde das Flackern nicht auftreten
        paintingComponent = new PaintingComponent();
        initWindow();
    }
        
    private void initWindow() {
        //einen JFrame erzeugen
        frame = new JFrame("Binrbaum");
        //Borderlayout whlen, dass das Fenster in die Bereiche Nord, Sd, Ost, West und Center unterteilt
        frame.setLayout(new BorderLayout());
        //dafr sorgen das das Programm beendet wird wenn man das 'X' anklickt
        frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        //eine JComponent hat keine Ahnung davon was man auf ihr zeichnen mchte.
        //Der LayoutManager hat also keine Mglichkeit die passende Gre fr unser
        //Objekt festzustellen und wrde von (0,0) ausgehen.
        //Daher helfen wir etwas nach und setzen die gewnschte Gre hndisch
        
        paintingComponent.setPreferredSize(new Dimension(2000,600)); 
        
        //Put the drawing area in a scroll pane.
        scrollPane = new JScrollPane(paintingComponent);
        scrollPane.setPreferredSize(new Dimension(800,300));
       

               
        
        //unsere Komponente wird mittig im JFrame plaziert 
        frame.add(scrollPane,BorderLayout.CENTER);
        //frame.setSize(500,400);
        
        //in den unteren Bereich des Frames packen wir einige 
        //Steuerelemente die wir der bersicht wegen in einer 
        //eigenen Methode erstellen und initialisieren
    
        JPanel panel = new JPanel(new FlowLayout());
        final JTextField tf = new JTextField(3);
        panel.add(tf);
        //noch ein Button zum Einfgen des Knotens
        JButton newNodeBtn = new JButton("Einfgen");
        panel.add(newNodeBtn);
        newNodeBtn.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                //Was soll gemacht werden, wenn einer auf den Button klickt???
                 try {
                     myTree.insert(Integer.parseInt(tf.getText()));
                     myTree.draw();
                    } catch (NumberFormatException nfe) {
                        System.out.println(tf.getText() + " ist keine Zahl.");   
                    }
            }
        
        });
        
        //noch ein Button zum Entfernen des Knotens
        JButton delNodeBtn = new JButton("Entfernen");
        panel.add(delNodeBtn);
        delNodeBtn.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                 try {
                     myTree.delete(Integer.parseInt(tf.getText()));
                     myTree.draw();
                    } catch (NumberFormatException nfe) {
                        System.out.println(tf.getText() + " ist keine Zahl.");   
                    }
            }
        
        });
        frame.add(panel,BorderLayout.SOUTH);    
        //der Frame enthlt nun alle bentigten Komponenten
        //und kann nun seine minimale Gre berechnen
        frame.pack();
        //und noch den Frame sichtbar machen und zentrieren
        frame.setVisible(true);
        frame.setLocationRelativeTo(null);
    }

    public void drawTree(BinBaum curTree, int tiefe) {
        myTree=curTree;
        paintingComponent.setRoot(myTree.getRoot());
        paintingComponent.setDepth(tiefe);
        paintingComponent.repaint();
    }
}
 
class PaintingComponent extends JComponent {
    private Baumelement root;
    private int depth;
    
    public PaintingComponent() {}

    public void setRoot(Baumelement newRoot) {
        this.root = newRoot;
    }

    public void setDepth(int newDepth) {
        this.depth = newDepth;
    }
    
    protected void paintComponent(Graphics g) {
        //Diese Methode wird immer ausgefhrt, wenn sich was am Fenster gendert hat oder soll
        super.paintComponent(g);
        Graphics2D g2d = (Graphics2D)g; 
        g2d.setRenderingHint( RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON); 
        //Jetzt muss man halt nur noch den Baum wandern und zeichnen,
        //d.h. ich hole mir die Knoten in preOrder w-LK-RK und zeichne die Elment
        if (root!=null) {
            int maxKnotenZahl = (int)Math.pow(2,root.tiefe());
            drawNode(g2d, root, 1000, 20, 1000, 20, root.tiefe(), maxKnotenZahl*60);
        }
    }
        
    public void wait(int milliseconds) { try {Thread.sleep(milliseconds);}  catch (Exception e){/*ignoring exception at the moment*/}}

    public void drawNode(Graphics2D g2d, Baumelement baumelement, int x, int y, int fatherX, int fatherY, int restTiefe, int maxWidth) {
        int strLength = baumelement.getData().length();
        int fontSize = 14;
        g2d.setFont(new Font("monospaced",Font.BOLD,fontSize));
        BasicStroke stroke = new BasicStroke(2.0f);
        g2d.setColor(Color.BLACK); 
        //Linie zum Vater
        if (fatherX!=x && fatherY!=y) { // Vaterknoten vorhanden
            g2d.drawLine(x+5, y, fatherX, fatherY);
        }
        g2d.fill(new Rectangle2D.Double (x-5,y-fontSize,15+strLength*5,fontSize+5));
        g2d.draw(new Rectangle2D.Double (x-5,y-fontSize,15+strLength*5,fontSize+5));
        g2d.setColor(Color.WHITE); 
        g2d.drawString(baumelement.getData(),x,y);
        if (baumelement.istKnoten()) {
            int xOffset = 30;
            if (restTiefe!=0) xOffset =(int)maxWidth/restTiefe;
            int yOffset = 30;
            int xFatherOffset = (strLength*5)/2;
            restTiefe--;
            if (baumelement.getRightChild().istAbschluss()) {
                drawNode(g2d, baumelement.getRightChild(), (int)(x+18), (int)(y+25), x+xFatherOffset, y, restTiefe,xOffset);    
            } else {
                drawNode(g2d, baumelement.getRightChild(), (int)(x+xOffset+30), (int)(y+yOffset), x+xFatherOffset, y, restTiefe,xOffset);
            }
            if (baumelement.getLeftChild().istAbschluss()) {
                drawNode(g2d, baumelement.getLeftChild(), (int)(x-18), (int)(y+25), x+xFatherOffset, y, restTiefe,xOffset);
            } else {
                drawNode(g2d, baumelement.getLeftChild(), (int)(x-xOffset-30), (int)(y+yOffset), x+xFatherOffset, y, restTiefe,xOffset);
            }
        }
    }
}